package mschmidt.komo;
/*
 * Custom action that controls the database reindex operation.
 */

import mschmidt.komo.datatools.UtilitySqlJob;

import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.actions.ActionFactory.IWorkbenchAction;
import mschmidt.komo.ApplicationImages;
import mschmidt.komo.preferences.PreferenceConstants;

public class DbReindexAction extends Action implements IWorkbenchAction {
	// Initialize variables for this action.
	private IWorkbenchWindow thisWindow;

	public final static String ACTION_ID = "mschmidt.komo.DbReindexAction";

	/*
	 * Create the action and set appearance variables.
	 */
	public DbReindexAction(final IWorkbenchWindow window) {
		thisWindow = window;
		setId(ACTION_ID);
		setText("Reindex Database");
		setToolTipText("Rebuild database indexes");
		setImageDescriptor(KomoPlugin
				.getImageDescriptor(ApplicationImages.DB_REINDEX_ICON));
	}

	public final void run() {
		if (thisWindow != null) {
			boolean retVal = MessageDialog.openQuestion(thisWindow.getShell(),
					"Confirm Reindex",
					"Reindexing may take several minutes\nProceed?");
			if (retVal) {
				if (Application.dbPassword.checkPassword()) {
					String stmt = "REINDEX DATABASE "
							+ KomoPlugin.getDefault().getPreferenceStore()
									.getString(PreferenceConstants.DB_NAME);
					Job reindexJob = new UtilitySqlJob("Reindex",
							ApplicationImages.DB_REINDEX_ICON, stmt);
					reindexJob.schedule();
				}
			}
		}
	}

	public void dispose() {
		// TODO Auto-generated method stub

	}

}
