package mschmidt.komo.datatools;

/*
 * Instances ot this object class provide a wizard page that allows the user to
 * input options for the restore operation.
 */

import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;

public class RestoreWizardPage extends WizardPage {

	private Combo restoreItemsCombo;

	private Text dropUtilityText;

	private Text restoreEngineText;

	private Text fileNameText;

	/**
	 * Constructor to create the wizard.
	 */
	public RestoreWizardPage() {
		super("restorePage");
		setTitle("Database Recovery Wizard");
		setDescription("Database Recovery Options");
		setPageComplete(false);
	}

	/**
	 * Create contents of the wizard.
	 * 
	 * @param parent
	 *            the composite's parent
	 */
	public final void createControl(final Composite parent) {
		Composite container = new Composite(parent, SWT.NULL);
		final GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 3;
		gridLayout.horizontalSpacing = 3;
		container.setLayout(gridLayout);
		//
		setControl(container);

		final Label archiveFileLabel = new Label(container, SWT.NONE);
		archiveFileLabel.setToolTipText("Select archive file to restore from");
		archiveFileLabel.setText("Restore File");

		fileNameText = new Text(container, SWT.BORDER);
		fileNameText.addModifyListener(new ModifyListener() {
			public void modifyText(final ModifyEvent e) {
				((RestoreWizard) getWizard()).data.restoreFile = fileNameText
						.getText();
				CheckPage();
			}
		});
		fileNameText.setLayoutData(new GridData(GridData.FILL, GridData.CENTER,
				true, false));
		fileNameText.setText(((RestoreWizard) getWizard()).data.restoreFile);

		Button fileBrowseButton = new Button(container, SWT.NONE);
		fileBrowseButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(final SelectionEvent e) {
				FileDialog dialog = new FileDialog(getShell(), SWT.OPEN);
				dialog.setFilterNames(new String[] {
						"PostgreSQL Custom Archive files (*.pgb)",
						"Bak Archive files (*.bak)",
						"Backup Archive files (*.backup)",
						"Tar Archive files (*.tar)", "All files (*.*)" });
				dialog.setFilterExtensions(new String[] { "*.pgb", "*.bak",
						"*.backup", "*.tar", "*.*" });
				String stReturn = dialog.open();
				if (stReturn != null && stReturn.length() > 0) {
					fileNameText.setText(stReturn);
				}
			}
		});
		fileBrowseButton.setText("Browse...");

		final Label restoreEngineLabel = new Label(container, SWT.NONE);
		restoreEngineLabel
				.setToolTipText("The PostgreSQL restore engine (pg_restore.exe)");
		restoreEngineLabel.setText("Restore Engine");

		restoreEngineText = new Text(container, SWT.BORDER);
		restoreEngineText.addModifyListener(new ModifyListener() {
			public void modifyText(final ModifyEvent e) {
				((RestoreWizard) getWizard()).data.restoreEngine = restoreEngineText
						.getText();
				CheckPage();
			}
		});
		restoreEngineText.setLayoutData(new GridData(GridData.FILL,
				GridData.CENTER, true, false));
		restoreEngineText
				.setText(((RestoreWizard) getWizard()).data.restoreEngine);

		final Button engineBrowseButton = new Button(container, SWT.NONE);
		engineBrowseButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(final SelectionEvent e) {
				FileDialog engineBrowseDialog = new FileDialog(getShell(),
						SWT.SAVE);
				engineBrowseDialog.setFilterNames(new String[] {
						"PostgreSQL restore engine (pg_restore.exe)",
						".exe files (*.exe)", "All files (*.*)" });
				engineBrowseDialog.setFilterExtensions(new String[] {
						"pg_restore.exe", "*.exe", "*.*" });
				String stReturn = engineBrowseDialog.open();
				if (stReturn != null && stReturn.length() > 0) {
					restoreEngineText.setText(stReturn);
					// Warn the user if pg_dump.exe was not selected.
					if (!stReturn.endsWith("pg_restore.exe")) {
						MessageDialog
								.openWarning(
										getShell(),
										"Restore Engine Warning",
										"The selected file does not appear to be\nthe PostgreSQL restore engine (pg_restore.exe)");
					}
				}

			}
		});
		engineBrowseButton.setText("Browse...");

		final Label dropUtilityLabel = new Label(container, SWT.NONE);
		dropUtilityLabel
				.setToolTipText("The PostgreSQL database delete utility (dropdb.exe)");
		dropUtilityLabel.setText("Delete Utility");

		dropUtilityText = new Text(container, SWT.BORDER);
		dropUtilityText.addModifyListener(new ModifyListener() {
			public void modifyText(final ModifyEvent e) {
				((RestoreWizard) getWizard()).data.dropUtil = dropUtilityText
						.getText();
				CheckPage();
			}
		});
		dropUtilityText.setLayoutData(new GridData(GridData.FILL,
				GridData.CENTER, true, false));
		dropUtilityText.setText(((RestoreWizard) getWizard()).data.dropUtil);

		final Button browseDropButton = new Button(container, SWT.NONE);
		browseDropButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(final SelectionEvent e) {
				FileDialog utilBrowseDialog = new FileDialog(getShell(),
						SWT.SAVE);
				utilBrowseDialog.setFilterNames(new String[] {
						"PostgreSQL delete utility (dropdb.exe)",
						".exe files (*.exe)", "All files (*.*)" });
				utilBrowseDialog.setFilterExtensions(new String[] {
						"dropdb.exe", "*.exe", "*.*" });
				String stReturn = utilBrowseDialog.open();
				if (stReturn != null && stReturn.length() > 0) {
					restoreEngineText.setText(stReturn);
					// Warn the user if dropdb.exe was not selected.
					if (!stReturn.endsWith("dropdb.exe")) {
						MessageDialog.openWarning(
							getShell(),
							"Database Delete Utility Warning",
							"The selected file does not appear to be\nthe PostgreSQL database delete utility (dropdb.exe)");
					}
				}

			}
		});
		browseDropButton.setText("Browse...");

		final Label restoreItemsLabel = new Label(container, SWT.NONE);
		restoreItemsLabel.setToolTipText("Select database items to restore");
		restoreItemsLabel.setText("Restore Items");

		restoreItemsCombo = new Combo(container, SWT.READ_ONLY);
		restoreItemsCombo.setItems(new String[] { "Database, drop and replace",
				"All, create new database first",
				"All, do not create database", "Data Only, triggers enabled",
				"Data Only, triggers disabled", "Schema Only" });
		restoreItemsCombo.setVisibleItemCount(6);
		restoreItemsCombo.setLayoutData(new GridData(GridData.FILL,
				GridData.CENTER, true, false));
		new Label(container, SWT.NONE);
		restoreItemsCombo
				.select(((RestoreWizard) getWizard()).data.itemsSelection);

		final Button cleanButton = new Button(container, SWT.CHECK);
		cleanButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(final SelectionEvent e) {
				((RestoreWizard) getWizard()).data.doClean = cleanButton
						.getSelection();
			}
		});
		cleanButton
				.setToolTipText("Check to delete items before restoring them");
		cleanButton.setText("Clean");

		final Button exitOnErrorButton = new Button(container, SWT.CHECK);
		exitOnErrorButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(final SelectionEvent e) {
				((RestoreWizard) getWizard()).data.doExitOnError = exitOnErrorButton
						.getSelection();
			}
		});
		exitOnErrorButton
				.setToolTipText("Check to terminate restore operation when an error is encountered");
		exitOnErrorButton.setText("Exit on error");
		new Label(container, SWT.NONE);
		if (restoreItemsCombo.getSelectionIndex() == 0 
				|| restoreItemsCombo.getSelectionIndex() == 1) {
			cleanButton.setSelection(false);
			cleanButton.setEnabled(false);
			exitOnErrorButton.setSelection(false);
			exitOnErrorButton.setEnabled(false);
		} else {
			cleanButton.setSelection(((RestoreWizard) getWizard()).data.doClean);
			cleanButton.setEnabled(true);
			exitOnErrorButton
				.setSelection(((RestoreWizard) getWizard()).data.doExitOnError);
			exitOnErrorButton.setEnabled(true);
		}

		final Button verboseButton = new Button(container, SWT.CHECK);
		verboseButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(final SelectionEvent e) {
				((RestoreWizard) getWizard()).data.doVerbose = verboseButton
						.getSelection();
			}
		});
		
		restoreItemsCombo.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(final SelectionEvent e) {
				((RestoreWizard) getWizard()).data.itemsSelection = restoreItemsCombo
						.getSelectionIndex();
				if (restoreItemsCombo.getSelectionIndex() == 0 
						|| restoreItemsCombo.getSelectionIndex() == 1) {
					cleanButton.setSelection(false);
					exitOnErrorButton.setSelection(false);
					cleanButton.setEnabled(false);
					exitOnErrorButton.setEnabled(false);
				} else {
					cleanButton.setEnabled(true);
					exitOnErrorButton.setEnabled(true);
				}
				CheckPage();
			}
		});
		
		verboseButton
				.setToolTipText("Check to display detailed information about Restore operation is completed");
		verboseButton.setText("Verbose");
		verboseButton
				.setSelection(((RestoreWizard) getWizard()).data.doVerbose);

		final Button ignoreVersionButton = new Button(container, SWT.CHECK);
		ignoreVersionButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(final SelectionEvent e) {
				((RestoreWizard) getWizard()).data.ignoreVersion = ignoreVersionButton
						.getSelection();
			}
		});
		ignoreVersionButton
				.setToolTipText("Check to ignore database version checks");
		ignoreVersionButton.setText("Ignore version");
		new Label(container, SWT.NONE);
		ignoreVersionButton
				.setSelection(((RestoreWizard) getWizard()).data.ignoreVersion);

		final Button analyzeButton = new Button(container, SWT.CHECK);
		analyzeButton.setLayoutData(new GridData(GridData.BEGINNING,
				GridData.CENTER, false, false, 2, 1));
		analyzeButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(final SelectionEvent e) {
				((RestoreWizard) getWizard()).data.doAnalyze = analyzeButton
						.getSelection();
			}
		});
		analyzeButton
				.setToolTipText("Check to perform an Analyze operation after the Restore operation (recommended)");
		analyzeButton.setText("Analyze after restore");
		analyzeButton
				.setSelection(((RestoreWizard) getWizard()).data.doAnalyze);
		new Label(container, SWT.NONE);
		new Label(container, SWT.NONE);
	}

	/*
	 * Method to check if page is complete.
	 */
	private void CheckPage() {
		if (((RestoreWizard) getWizard()).data.restoreEngine != null
				&& ((RestoreWizard) getWizard()).data.restoreEngine.length() > 0
				&& ((RestoreWizard) getWizard()).data.restoreFile != null
				&& ((RestoreWizard) getWizard()).data.restoreFile.length() > 0
				&& ((RestoreWizard) getWizard()).data.dropUtil != null
				&& ((RestoreWizard) getWizard()).data.dropUtil.length() > 0
				&& ((RestoreWizard) getWizard()).data.itemsSelection > -1
				&& ((RestoreWizard) getWizard()).data.itemsSelection < 6) {
			setPageComplete(true);
		} else {
			setPageComplete(false);
		}

	}
}
